#!/usr/bin/python
# -*- coding: UTF-8 -*-

#
# Copyright (C) 2007 by Hernán Ordiales
# <hordia@uint8.com.ar>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

# follow commits: A program to keep informed about project status

import os, sys
from PyQt4.QtGui import *
from PyQt4.QtCore import *
from xmlrpclib import ServerProxy
import httplib, feedparser

true = 1
false = 0

class MainWindow(QMainWindow):
	
	def __init__( self ):
		QMainWindow.__init__( self )
		self.title = "follow clam"
		self.version = "0.1"
		self.setWindowTitle( self.title + ' ' + self.version )
		icon = QIcon( "logo.xpm" )
		self.setWindowIcon( icon )

		#Here default values, then overridden by config file
		self.conf = {}
		self.conf['default_width'] = 600
		self.conf['default_height'] = 410
		self.conf['config_file'] = '.config'
		self.conf['clam_feed'] =  "http://cia.vc/stats/project/clam/.rss"
		self.conf['last_revision'] = 0
		self.conf['enable_notifications'] = 'true'

		group_box = QGroupBox( self )
		v_layout = QVBoxLayout( group_box )
		group_box.setLayout( v_layout )

		self.overview_msg = QTextBrowser( self )
		self.overview_msg.setOpenExternalLinks( true )
		
		self.exit_button = QPushButton( "Close", self )
		self.exit_button.setDefault( true )
		self.h_endlayout = QHBoxLayout()
		self.h_endlayout.addItem( QSpacerItem(10, 10, QSizePolicy.Expanding, QSizePolicy.Minimum) )
		self.h_endlayout.addWidget( self.exit_button )
		self.h_endlayout.addItem( QSpacerItem(10, 10, QSizePolicy.Expanding, QSizePolicy.Minimum) )

		v_layout.addWidget( self.overview_msg )
		v_layout.addItem( self.h_endlayout )
		self.setCentralWidget( group_box )

		self.connect( self.exit_button, SIGNAL("clicked()"), self.on_close )

		#timer
		self.timer = QTimer( self )
		self.conf['update_time'] = 10 * (60*1000) #10 min
		self.connect( self.timer, SIGNAL("timeout()"), self.update )
		self.timer.start( int(self.conf['update_time']) )
		
		#system tray
		self.systray = QSystemTrayIcon( self )
		self.systray.setIcon( QIcon("logo.xpm") )
		self.systray_menu = QMenu( self.title )
		self.systray_update = QAction( "Update", self )
		self.systray_exit = QAction( "Exit", self )
		self.systray_menu.addAction( self.systray_update )
		self.systray_menu.addAction( self.systray_exit )
		self.systray.setContextMenu( self.systray_menu )
		self.connect( self.systray, SIGNAL("activated(QSystemTrayIcon::ActivationReason)"), self.systray_clicked )
		self.connect( self.systray_update, SIGNAL("triggered()"), self.update )
		self.connect( self.systray_exit, SIGNAL("triggered()"), self.on_exit )
		self.systray.show()

	def closeEvent( self, QCloseEvent ):
		#self.systray.show()
		self.hide()

	def on_close( self ):
		self.hide()

	def on_exit( self ):
		print "exiting..."
		self.conf['default_width'] = self.width()
		self.conf['default_height'] = self.height()
		self.save_config()
		QApplication.instance().exit()

	def systray_clicked( self ):
		self.show()
		#self.systray.hide()

	def init_config( self ):
		#if not os.path.isdir('cache'): os.mkdir( 'cache' )
		if not self.load_config():
			print "Error loading config"
			return false

		self.resize( int(self.conf['default_width']), int(self.conf['default_height']) )
		
		self.read_feed()
		return true

	def load_config( self ):
		try:
			f = open( self.conf['config_file'] , 'r' )
		except IOError:
			return self.save_config()

		#reads every property from config file
		temp = f.read()
		f.close
		for elem in temp.split('\n'):
			if elem!='' and elem[0]!='#':
				(item, value) = elem.split(',')
				self.conf[ item ] = value
		return true

	def save_config( self ):
		try:
			g = open( self.conf['config_file'] , "w" )
		except:
			print "error opening config file"
			return false

		print "saving every property to config file"
		for item in self.conf.keys():
			g.write( str(item) + ',' + str(self.conf[item]) + '\n' )
		g.close()
		return true

	def update( self ):
		if self.isVisible() and self.conf['enable_notifications']=='true':
			self.systray.showMessage( self.title, "updating feed...", QSystemTrayIcon.Information, 10000 )
		self.overview_msg.clear()
		html_result = self.read_feed()
		#self.overview_msg.insertHtml( html_result )
		
	def read_feed( self ):
		print "updating feed..."
		output = ""
		feed = feedparser.parse( self.conf['clam_feed'] )
		
		#new commits?
		try:
			tmp_text = QTextEdit()
			tmp_text.insertHtml( feed['entries'][0].description )
			last_revision = ( (tmp_text.toPlainText().split('\n')[0]).split(':')[2] ).split('/')[0][2:]
			print "Last revision:",last_revision
			if self.conf['last_revision'] != last_revision:
				print "new commits!"
				self.systray.showMessage( self.title, "New commits!", QSystemTrayIcon.Information, 10000 )
				self.conf['last_revision'] = last_revision
		except:
			print "Item error"
		
		for item in feed['entries']:
			#print item.title
			#output += item.title + '<br />'
			output += str(item.description)

		#FIXME because output return no works (and i don't know why) i have to update overview here
		self.overview_msg.insertHtml( output )

		return output


app = QApplication( sys.argv )
main_window = MainWindow()
main_window.show()
if main_window.init_config():
	sys.exit( app.exec_() )
